#include <osgProducer/Viewer>
#include <osgGA/TrackballManipulator>
#include <osg/Drawable>
#include <osg/Timer>

#include <osgCal/CoreModel>
#include <osgCal/Model>

#define DIR "data"

class MyManipulator: public osgGA::TrackballManipulator {
public:
    MyManipulator(osgCal::Model *model) {
	this->model=model;
        nMov=0;
    }
    /** handle events, return true if handled, false otherwise.*/
    virtual bool handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter& us) {
	if (ea.getEventType() == osgGA::GUIEventAdapter::KEYDOWN) {
	    if (ea.getKey() == 'm') {
		model->stop(nMov, 3);
		nMov = ++nMov%6;
		printf("Movement No.%d (Blend time 3sec)\n", nMov);
		model->loop(nMov, 1, 3);
		return true;
	    }
	}
	return TrackballManipulator::handle(ea,us);
    }
private:
    osg::ref_ptr<osgCal::Model> model;
    int nMov;
};

void load(osgCal::CoreModel *core) {
    core->loadSkeleton(DIR"/paladin.csf");
    core->loadAnimation(DIR"/paladin_walk.caf");
    core->loadAnimation(DIR"/paladin_wave.caf");
    core->loadAnimation(DIR"/paladin_strut.caf");
    core->loadAnimation(DIR"/paladin_sneak.caf");
    core->loadAnimation(DIR"/paladin_shoot_arrow.caf");
    core->loadAnimation(DIR"/paladin_jog.caf");
    core->loadMesh(DIR"/paladin_body.cmf");
    core->loadMesh(DIR"/paladin_cape.cmf");
    core->loadMesh(DIR"/paladin_loincloth.cmf");
    core->loadMaterial(DIR"/paladin_cape.crf");
    core->loadMaterial(DIR"/paladin_head.crf");
    core->loadMaterial(DIR"/paladin_chest.crf");
    core->loadMaterial(DIR"/paladin_gird.crf");
    core->loadMaterial(DIR"/paladin_arm_left.crf");
    core->loadMaterial(DIR"/paladin_arm_right.crf");
    core->loadMaterial(DIR"/paladin_flesh.crf");
    core->loadMaterial(DIR"/paladin_edge.crf");
    core->loadMaterial(DIR"/paladin_stripe.crf");
    core->loadMaterial(DIR"/paladin_loin_upper.crf");
    core->loadMaterial(DIR"/paladin_loin_lower.crf");
    core->loadMaterial(DIR"/paladin_legplate_right.crf");
    core->loadMaterial(DIR"/paladin_legplate_left.crf");
    core->loadMaterial(DIR"/paladin_leg_right.crf");
    core->loadMaterial(DIR"/paladin_leg_left.crf");
    core->loadMaterial(DIR"/paladin_legguard_right.crf");
    core->loadMaterial(DIR"/paladin_legguard_left.crf");
    core->loadMaterial(DIR"/paladin_shoe.crf");
    core->loadMaterial(DIR"/paladin_ponytail.crf");
    core->loadAllTextures(DIR);
}

int main() {
    //////// osgCal stuff //////////
    // Creating a core model, the template
    osgCal::CoreModel *core = new osgCal::CoreModel();
    load(core); //< loading some data into it

    // Creating a concrete model using the core template
    osgCal::Model *model = new osgCal::Model();
    model->create(core);

    // Setting the first animation in loop mode, weight 1, and starting just now
    model->loop(0, 1.0f, 0.0f);

    //////// osgProducer stuff /////////

    osgProducer::Viewer viewer;
    // set up the value with sensible default event handlers.
    viewer.setUpViewer(osgProducer::Viewer::STANDARD_SETTINGS);
    viewer.setSceneData(model);
    unsigned int pos = viewer.addCameraManipulator(new MyManipulator(model));
    viewer.selectCameraManipulator(pos);
    viewer.realize();
    while ( !viewer.done() ) {
	viewer.sync();
	viewer.update();
        viewer.frame();
    }
    viewer.sync();
    return 0;
}

