/*
 *
 * Copyright (C) 2004 Mekensleep
 *
 *	Mekensleep
 *	24 rue vieille du temple
 *	75004 Paris
 *       licensing@mekensleep.com
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *  Cedric PINSON <cpinson@freesheep.org>
 *  Loic Dachary <loic@gnu.org>
 *
 */

#include <cal3d/cal3d.h>

#include <osgCal/SubMeshSoftware>

#include <iostream>

using namespace osgCal;

SubMeshSoftware::SubMeshSoftware() {
  std::cerr << "SubMesh::SubMesh(): You should never call this constructor!" << std::endl;
}

SubMeshSoftware::SubMeshSoftware(CalSubmesh* calSubmesh, CalRenderer* calRenderer) :
  _supportsPrimitiveFunctor(true),
  _invisible(false),
  _calSubmesh(calSubmesh),
  _calRenderer(calRenderer)
{
  setSupportsDisplayList(false);
}

SubMeshSoftware::SubMeshSoftware(const SubMeshSoftware& submesh, const osg::CopyOp& copyop) :
  osg::Geometry(submesh, copyop),
  _supportsPrimitiveFunctor(submesh._supportsPrimitiveFunctor),
  _invisible(submesh._invisible),
  _calSubmesh(submesh._calSubmesh),
  _calRenderer(submesh._calRenderer)
{
}

SubMeshSoftware::~SubMeshSoftware() {}

void SubMeshSoftware::update(void)
{
  _calRenderer->m_pSelectedSubmesh = _calSubmesh;

  // vertexes
  {
    osg::Vec3Array* array = dynamic_cast<osg::Vec3Array*>(getVertexArray());
    int v=_calRenderer->getVertices((float*)array->getDataPointer());
  }

  // normals
  if(!_invisible) {
    osg::Vec3Array* array = dynamic_cast<osg::Vec3Array*>(getNormalArray());
    _calRenderer->getNormals((float*)array->getDataPointer());
  }

  dirtyBound();
  dirtyDisplayList();
}

void SubMeshSoftware::drawImplementation(osg::State& state) const 
{
  if(!_invisible) Geometry::drawImplementation(state);
}

bool SubMeshSoftware::computeBound() const
{
  if(_supportsPrimitiveFunctor)
    return Geometry::computeBound();
  else {
    _bbox_computed = true;
    return true;
  }
}

void SubMeshSoftware::create(void)
{
  _calRenderer->m_pSelectedSubmesh = _calSubmesh;

  // vertexes
  {
    int count = _calRenderer->getVertexCount();
    osg::Vec3Array* array = new osg::Vec3Array(count);
    _calRenderer->getVertices((float*)array->getDataPointer());
    setVertexArray(array);
  }

  // normals
  if(!_invisible) {
    int count = _calRenderer->getVertexCount();
    osg::Vec3Array* array = new osg::Vec3Array(count);
    _calRenderer->getNormals((float*)array->getDataPointer());
    setNormalArray(array);
    setNormalBinding(osg::Geometry::BIND_PER_VERTEX);
  }

  // texture coordinates
  if(!_invisible) {
    int count = _calRenderer->getVertexCount();
    for(int mapId = 0; mapId < _calRenderer->getMapCount(); mapId++) {
      osg::Vec2Array* array = new osg::Vec2Array(count);
      _calRenderer->getTextureCoordinates(mapId, (float*)array->getDataPointer());
      setTexCoordArray(mapId, array);
    }
  }

  // faces
  {
    int count = _calRenderer->getFaceCount();
    osg::PrimitiveSet* primitive;
    if(sizeof(CalIndex) == sizeof(GLushort)) {
      osg::DrawElementsUShort* primitiveUShort = new osg::DrawElementsUShort(osg::PrimitiveSet::TRIANGLES, count*3);
      primitive = primitiveUShort;
      _calRenderer->getFaces((CalIndex*)&(primitiveUShort->front()));
    } else {
      osg::DrawElementsUInt* primitiveUInt = new osg::DrawElementsUInt(osg::PrimitiveSet::TRIANGLES, count*3);
      primitive = primitiveUInt;
      _calRenderer->getFaces((CalIndex*)&(primitiveUInt->front()));
    }
    addPrimitiveSet(primitive);
  }
}

